#!/usr/bin/perl
#
# $Revision: 1.99 $
# Copyright 2009 Teleflora.
#
# Apply updates to the underlying Linux operating system.
# Primarily:
#	Apply redhat errata.
#	Register with redhat
#	(Un) Register with redhat.
#	Upgrade packages as needed for either RTI v14 or Daisy 8.
#

use strict;
use warnings;
use English;
use Getopt::Long;
use POSIX;


my $HELP = 0;
my $VERSION = 0;
my $BAREMETAL = 0;
my $RTIV12 = 0;
my $RTIV14 = 0;
my $DAISYV8 = 0;
my $OSPATCHES = 0;
my $RHNREG = 0;
my $REBOOT = 0;
my $SWAPON = 0;
my $FORENSICS = 0;
my $OSTOOLS = 0;
my $ONETIMEROOT = 1;
my $IPADDR = "";
my $TFSERVER = "rtihardware.homelinux.com";

my $CUPSTMP = 0;
my $CUPSCONF = 0;
my $CUPSCONF_MAXJOBS = 1;
my $CUPSCONF_DISABLETEMPFILES = 1;
my $CUPSCONF_TIMEOUTS = 1;
my $CUPSCONF_ERRORPOLICY = 1;
my $PURGEPRINT = 0;
my $PURGERPMS = 0;
my $HARDWARE = "";
my $OS = "";
my $ARCH = "";
my $POSDIR = "";
my $BBJ = "2145830.jar.gz";
my $BLM = "blm.2145830.jar.gz";
my $JRE_X64 = "jre-6u16-linux-x64-rpm.bin"; # Get update 16 from java.com
my $JRE_X86 = "jre-6u16-linux-i586-rpm.bin";
my $pid = 0;


GetOptions(
	"help" => \$HELP,
	"version" => \$VERSION,
	"baremetal" => \$BAREMETAL,
	"rti12" => \$RTIV12,
	"rti14" => \$RTIV14,
	"daisy8" => \$DAISYV8,
	"cupstmp" => \$CUPSTMP,
	"cupsconf" => \$CUPSCONF,
	"purgeprint" => \$PURGEPRINT,
	"purgerpms" => \$PURGERPMS,
	"ospatches" => \$OSPATCHES,
	"ostools" => \$OSTOOLS,
	"rhnreg" => \$RHNREG,
	"hardboot" => \$REBOOT,
	"reboot" => \$REBOOT,
	"swapon" => \$SWAPON,
	"forensics" => \$FORENSICS,
	"onetimeroot" => \$ONETIMEROOT,
	"ipaddr=s" => \$IPADDR,
	"tfserver=s" => \$TFSERVER,
);



# --version
if($VERSION != 0) {
	print("OSTools Version: 1.1\n");
	print("$0 " . '$Revision: 1.99 $' . "\n");
	exit(0);
}

# --help
if($HELP != 0) {
	usage();
	exit(0);
}


# Which Processor are we using?
open(PIPE, "uname -i |");
while(<PIPE>) {
	if(/i.86/) {
		$ARCH = "x86";
	}
	if(/x86_64/) {
		$ARCH = "x64";
	}
}
close(PIPE);

# WHich OS are we using?
open(FILE, "< /etc/redhat-release");
while(<FILE>) {
	if(/release 4/) {
		$OS = "RHEL4";
	}
	if(/release 5/) {
		$OS = "RHWS5";
	}
}
close(FILE);


if (-d "/d/daisy") {
	$POSDIR = "/d/daisy";
} else {
	$POSDIR = "/usr2/bbx";
}


# We must be root to do these things.
if($EUID == 0) {

	loginfo("Starting updateos.pl " . '$Revision: 1.99 $' . "\n");
	# --rhnreg
	if($RHNREG != 0) {
		register_redhat();
	}

	# --baremetal
	if($BAREMETAL != 0) {
		print("Using Teleflora Package Server: \"$TFSERVER\"\n");
		setup_baremetal_packages();
	}

	# --ospatches
	if($OSPATCHES != 0) {
		update_os_patches();
	}


	# --ostools
	if($OSPATCHES != 0) {
		exit(update_ostools());
	}

	# --rti14
	if($RTIV14 != 0) {
		install_rtiv14_packages();
	}


	# --rti12
	if($RTIV12 != 0) {
		install_rtiv12_packages();
	}


	# --daisy8
	if($DAISYV8 != 0) {
		install_daisyv8_packages();
	}

	# --ipaddr=x.x.x.x
	if ($IPADDR) {
		configure_ip_addr($IPADDR);
	}


	# --cupstmp
	if ($CUPSTMP != 0) {
		cups_clean_tempfiles();
	}

	# --cupconf
	if ($CUPSCONF != 0) {
		cups_edit_conf();
	}

	# --purgeprint
	if($PURGEPRINT != 0) {
		purge_printjobs();
	}

	# --purgerpms
	if($PURGERPMS != 0) {
		purge_rpms();
	}

	# --hardboot
	# --reboot
	if($REBOOT != 0) {
		exit(hard_reboot());
	}

	# --swapon
	if($SWAPON != 0) {
		exit(swapon());
	}

	# --forensics
	if($FORENSICS != 0) {
		exit(forensics());
	}
}



exit(0);
###################################################
###################################################
###################################################


sub usage
{
	print(<< "EOF");
$0 Usage:

$0 --version
$0 --help
$0 --baremetal
$0 --rti12 [--tfserver=rtihardware.homelinux.com]
$0 --rti14 [--tfserver=rtihardware.homelinux.com]
$0 --daisy8 [--tfserver=rtihardware.homelinux.com]
$0 --ipaddr=192.168.1.x
$0 --cupstmp
$0 --cupsconf
$0 --purgeprint
$0 --purgerpms
$0 --ospatches
$0 --ostools
$0 --rhnreg
$0 --reboot
$0 --swapon
$0 --forensics > /tmp/forensics.txt
EOF

}



# Register with the redhat portal.
# Make sure our rhn daemon is enabled.
sub register_redhat
{
	my $temp = "";
	my $svctag = "";
	my $vendor = "";
	my $hardware = "";


	system("grep Cent /etc/redhat-release > /dev/null 2> /dev/null");
	if($? == 0) {
		print("CentOS does not need to register.\n");
		return(0);
	}


	# Get our service tag number
	# Get the "type of" hardware we are on. 
	# (Note this could be inaccurate if we bare-metal restore to a different box.)
	open(PIPE, "/usr/sbin/dmidecode | grep -A 5 \"System Information\" |");
	while(<PIPE>) {
		chomp;
		if(/(Manufacturer:)(\s+)(\S+)/) {
			$vendor = $3;
		}
		if(/(Serial Number:)(\s+)(\S+)/) {
			$svctag = uc($3);
		}
		if(/(Product Name:)([[:print:]]+)([\s*\d+\s*]+)$/) {
			$hardware = $2;
		}

	}
	close(PIPE);


	loginfo("Obtaining a Redhat License...");
	$temp = $svctag;
	if("$hardware" ne "") {
		$temp .= " $hardware";
	}
	system("/usr/sbin/rhnreg_ks --force --profilename \"$svctag $vendor $hardware\" --activationkey=0ad77379739f3e7b6c3263070a1ab0dc");


	foreach $temp ("rhnsd", "yum-updatesd") {
		if(-f "/etc/rc.d/init.d/$temp") {
			system("/sbin/chkconfig --level 3 $temp on");
			system("/sbin/chkconfig --level 5 $temp on");
		}
	}
}


#
# Sort of a "post install" set of steps for a new Dell t3500 server.
#
sub setup_baremetal_packages
{

	register_redhat();

	remount_partition("/", "/");
	remount_partition("SWAP-sda2", "swap");
	remount_partition("SWAP-sda3", "swap");
	remount_partition("SWAP-sda4", "swap");

	loginfo("Installing Additional Redhat software.");
	identify_hardware();
	system("yum install -y nmap");


	# Grab Poweredge 
	if("$HARDWARE" eq "t300") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		system("wget --proxy=off --cache=off -O /tmp/delldset-poweredge.bin http://$TFSERVER/ks/delldset-poweredge.bin");
		system("chmod +rx /tmp/delldset-poweredge.bin");
		# Here we should run the script and feed the prompts given to us.
	}

	if("$HARDWARE" eq "420") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "1800") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}

	if("$HARDWARE" eq "t3400") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "t3500") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "precision380") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "precision390") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "2950") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}
	if("$HARDWARE" eq "r910") {
		# Grab BIOS updates
		# Grab RAID controller firmware updates
		# Grab Dell "DSET".
		# Here we should run the script and feed the prompts given to us.
	}


	# Require root to change password.
	if ($ONETIMEROOT) {
		system("/usr/sbin/usermod -L root");
		system("chage -d 0 root");
		system("/usr/sbin/usermod -U root");
	}
}


#
# Modify system config file: /etc/sysconfig/network-scripts/ifcfg-eth0
#
# change BOOTPROTO=dhcp --> BOOTPROTO=static
# add IPADDR=192.168.1.21
# add NETMASK=255.255.255.0
# add GATEWAY=192.168.1.1
#
sub configure_ip_addr
{
	my $ipaddr = $_[0];
	my $conf_file = '/etc/sysconfig/network-scripts/ifcfg-eth0';
	my $new_conf_file = "$conf_file.$$";

	if ($ipaddr !~ /^([\d]+)\.([\d]+)\.([\d]+)\.([\d]+)$/) {
		logerror("Error: invalid: $ipaddr\n");
		return;
	}

	my $is_invalid = 0;
	foreach my $s (($1, $2, $3, $4)) {
		if ($1 != 192 || $2 != 168 || $3 != 1 || $4 < 2 || $4 > 254) {
			$is_invalid = 1;
			last;
		}
	}
	if ($is_invalid) {
		logerror("Error: invalid: $ipaddr\n");
		return;
	}

	unless (-f "$conf_file") {
		logerror("Error: $conf_file does not exist.");
		return;
	}

	unless (open(OLD, "< $conf_file")) {
		logerror("Error: Could not open $conf_file for read.");
		return;
	}

	unless (open(NEW, "> $new_conf_file")) {
		logerror("Error: Could not open $new_conf_file for write.");
		close(OLD);
		return;
	}

	my $is_already_static = 0;
	while (<OLD>) {
		if (/^BOOTPROTO=static/) {
			$is_already_static = 1;
		} elsif (/^BOOTPROTO=dhcp/) {
			print(NEW "BOOTPROTO=static\n");
		} else {
			# By default, just copy what we read.
			print(NEW $_);
		}
	}

	close(OLD);
	close(NEW);

	if ($is_already_static) {
		logerror("System already configured for static IP address.");
		system("rm $new_conf_file");
		return;
	}

	# It's an error if the new conf file is zero length at this point.
	if (-z $new_conf_file) {
		logerror("Error: new conf file is zero length.");
		system("rm $new_conf_file");
		return;
	}

	# Open new conf file again to append some more lines.
	unless (open(NEW, ">> $new_conf_file")) {
		logerror("Error: Could not open $new_conf_file for append.");
		close(OLD);
		return;
	}

	print(NEW << "EOF");
IPADDR=$ipaddr
NETMASK=255.255.255.0
GATEWAY=192.168.1.1
EOF
	close(NEW);

	# Assume conf file was successfully transformed... so replace the old one with the new.
	system("chmod --reference=$conf_file $new_conf_file");
	system("chown --reference=$conf_file $new_conf_file");
	system("mv $new_conf_file $conf_file");

	print("Reboot or run \"# service network restart\" to activate network changes.\n");
}


sub update_os_patches
{

	loginfo("Installing OS Patches...");


	if($OS eq "RHEL4") {
		system("up2date --update");
	} else {
		system("yum clean -y all");
		system("yum update -y");
	}

	reset_aide_database();
}


# Just grab and run an install script from our website.
# Let that script do the hard work.
sub update_ostools
{
	print("Updating Teleflora OS Tools...\n");
	system("wget -O - http://$TFSERVER/ks/install-ostools.pl | perl - --update");
	return($?);
}


#
# Remove older kernels except three most recent.
# This to work around undersized /boot partitions on our older
# RHEL4 and RHWS5 machines.
#
sub purge_rpms
{
	my $thiskernel = "";
	my $thisrpm = "";
	my $thisfile = "";
	my $keepcount = 0;

	loginfo("Removing Old Kernels...");


	# Keep the three most recent kernels.
	# Remove everything else.
	foreach $thisfile("/boot/vmlinuz-*", "/usr/src/kernels") {


		open(FILES, "ls -t $thisfile 2>/dev/null |");
		while(<FILES>) {
			chomp;
			$thiskernel = $_;
			next if "$thiskernel" eq ".";
			next if "$thiskernel" eq "..";

			$keepcount++;
			next if($keepcount < 4);
			open(RPM, "rpm -qf $thiskernel |");
			while(<RPM>) {
				chomp;
				$thisrpm = $_;
				loginfo("Removing Kernel  $thisrpm...");
				print ("Removing Kernel  $thisrpm...\n");
				system("rpm --erase $thisrpm");
			}
			close(RPM);
		}
		close(FILES);
	}



	# Any other RPM packages which we should delete here?
	# eg. 'X', GCC, etc.


	return(0);
}






#
# Install OS pre-requisites for RTI v14.
#
sub install_rtiv14_packages
{
	my $bbj_nozip = "";
	my $line = "";

	loginfo("Installing RTI v14 Software Dependencies...");
	loginfo("Using Teleflora Package Server \"$TFSERVER\"");


	remount_partition("/teleflora", "/usr2");

	# Install Fetchmail
	system("yum -y install fetchmail");
	

	# Install apache
	loginfo("Installing Apache...");
	if("$OS" eq "RHWS5") {
		system("yum -y install httpd");
	}
	if("$OS" eq "RHEL4") {
		system("up2date --install httpd");
	}


	# Install Java
	loginfo("Installing Java JRE...");
	if("$ARCH" eq "x86") {
		if(! -f "/tmp/$JRE_X86") {
			system("wget --proxy=off --cache=off -O /tmp/$JRE_X86 http://$TFSERVER/ks/$JRE_X86");
		}
		if(! -f "/tmp/$JRE_X86") {
			logerror("Could not download file http://$TFSERVER/ks/$JRE_X86");
		} else {
			system("chmod a+rx /tmp/$JRE_X86");
			system("/tmp/$JRE_X86");
		}

	} else {
		system("rpm -ivh \"http://$TFSERVER/ks/ksh-20080202-14.el5.x86_64.rpm\"");
		if(! -f "/tmp/$JRE_X64") {
			system("wget --proxy=off --cache=off -O /tmp/$JRE_X64 http://$TFSERVER/ks/$JRE_X64");
		}
		if(! -f "/tmp/$JRE_X64") {
			logerror("Could not download file http://$TFSERVER/ks/$JRE_X64");
		} else {
			system("chmod a+rx /tmp/$JRE_X64");
			system("/tmp/$JRE_X64");
		}
	}
	# Link correct java
	system("rm -f /usr/bin/java");
	system("ln -s /usr/java/latest/bin/java /usr/bin/java");

	# Install BBj
	if(-d "/usr2/basis") {
		loginfo("Moving current /usr2/basis/ to /usr2/basis-backup/...");
		system("mv /usr2/basis /usr2/basis-backup");
	} else {
		loginfo("/usr2/basis/ did not previously exist...");
	}


	
	loginfo("Installing BBj...");
	if(! -f "/tmp/$BBJ") {
		system("wget --proxy=off --cache=off -O /tmp/$BBJ http://$TFSERVER/ks/$BBJ");
		if(! -f "/tmp/$BBJ") {
			logerror("Could not download file http://$TFSERVER/ks/$BBJ");
		}
	}

	#Create bbjinstallsettings.txt
	open(BBJINST, "> /tmp/bbjinstallsettings.txt");
        print(BBJINST << 'EOF');
-V LICENSE_ACCEPT_BUTTON="true"
-V LICENSE_REJECT_BUTTON="false"
-P installLocation="/usr2/basis/"
-V IS_SELECTED_INSTALLATION_TYPE=custom
-P BBjFtrBean.active=true
-P BBjThinClientFtr_bean.active=true
-P BBjSvcs_FtrBean.active=true
-P BBjEUTFtr_bean.active=true
-P EMFeatureBean.active=true
-P adminFeatureBean.active=true
-P BBjUtilsFtr_bean.active=true
-P GMLFeatureBean.active=true
-P BWUFeatureBean.active=true
-P STDFeatureBean.active=true
-P EXTFeatureBean.active=true
-P MKRecoverFeatureBean.active=true
-P AutoLicFeatureBean.active=true
-P CLlibFeatureBean.active=true
-P JLlibFeatureBean.active=true
-P BBjDevelToolsFtr_bean.active=true
-P perfAnalyzerFeatureBean.active=false
-P compilerListerFeatureBean.active=true
-P configuratorFeatureBean.active=false
-P guiBuilderFeatureBean.active=false
-P blmFeatureBean.active=true
-P BLMadminFeatureBn.active=true
-P basisIDEfeatureBean.active=false
-P documentationFeatureBean.active=false
-P jdbcFeatureBean.active=true
-P TrainingfeatureBean.active=true
-P DemosBean.active=true
-P BaristafeatureBean.active=false
-P AddOnfeatureBean.active=false
EOF
	close(BBJINST);
	if( (-f "/tmp/$BBJ")
	&&  (-f "/tmp/bbjinstallsettings.txt")
	) {
		$bbj_nozip = $BBJ;
		$bbj_nozip =~ s/\.gz//g; # blah.jar.gz -> blah.jar
		system("gzip -dc /tmp/$BBJ > /tmp/$bbj_nozip");
		system("java -jar /tmp/$bbj_nozip -options /tmp/bbjinstallsettings.txt -silent");
	}

	# Update BBj.properties File
	if(-f "/usr2/basis/cfg/BBj.properties") {
		loginfo("Backing up BBj.properties");
		system("mv /usr2/basis/cfg/BBj.properties /usr2/basis/cfg/BBj.properties.original");
	}
	loginfo("Updating BBj.properties");
	open(BBJPROP, "> /usr2/basis/cfg/BBj.properties");
	print(BBJPROP << 'EOF');
#BBj Services Properties
#Thu Aug 27 11:42:54 GMT-05:00 2009
basis.cacheDirectory=/usr2/basis/cache
basis.java.args.BBjCpl=-XX\:NewSize\=24m -client
basis.java.args.BBjLst=-XX\:NewSize\=24m -client
basis.java.args.BBjServices=-Xmx512m -Xms256m -XX\:MaxPermSize\=96m -XX\:NewSize\=64m -XX\:+UseConcMarkSweepGC -XX\:+CMSIncrementalMode -XX\:CMSIncrementalSafetyFactor\=20 -XX\:CMSInitiatingOccupancyFraction\=70 -XX\:MaxTenuringThreshold\=4 -XX\:SurvivorRatio\=14 -server -XX\:CompileCommandFile\=/usr2/basis/cfg/.hotspot_compiler -Dnetworkaddress.cache.ttl\=10 -Dsun.net.inetaddr.ttl\=10 -Djava.awt.headless\=true -verbose\:gc -Xloggc\:/usr2/basis/log/gc.log -XX\:+PrintGCDetails -XX\:+PrintGCTimeStamps -XX\:+PrintGCApplicationConcurrentTime -XX\:+PrintGCApplicationStoppedTime
basis.java.args.BasisIDE=-XX\:CompileCommandFile\=/usr2/basis//cfg/.hotspot_compiler
basis.java.args.Default=-Xmx128m -Xms128m -XX\:NewRatio\=4 -XX\:MaxPermSize\=128m -XX\:NewSize\=24m -client -XX\:CompileCommandFile\=/usr2/basis//cfg/.hotspot_compiler
basis.java.classpath=/usr2/basis//lib/BBjUtil.jar\:/usr2/basis//lib/BBjIndex.jar\:/usr2/basis//lib/FontChooser.jar\:/usr2/basis//lib/DemoClientFiles.jar\:/usr2/basis//lib/ClientObjects.jar\:/usr2/basis//lib/examples.jar\:/usr2/basis//lib/jfreechart-experimental.jar\:/usr2/basis//lib/jna.jar\:/usr2/basis//lib/swingx.jar\:/usr2/basis//lib/TimingFramework.jar\:/usr/java/latest/lib/ext/jdic_stub_unix.jar
basis.java.jvm.BBjServices=/usr/java/latest/bin/java
basis.java.jvm.Default=/usr/java/latest/bin/java
basis.java.skin=
basis.pdf.fontpath=
com.basis.bbj.bridge.BridgeServer.bindAddr=0.0.0.0\:2007
com.basis.bbj.bridge.BridgeServer.maxClients=100
com.basis.bbj.bridge.BridgeServer.start=false
com.basis.bbj.comm.FacadeFactory.suppressClientObjErrorsOnVoidMethods=true
com.basis.bbj.comm.InterpreterServer.backLog=110
com.basis.bbj.comm.InterpreterServer.bindAddr=127.0.0.1\:2005
com.basis.bbj.comm.InterpreterServer.maxClients=500
com.basis.bbj.comm.InterpreterServer.start=false
com.basis.bbj.comm.PortRequestServer.bindAddr=127.0.0.1\:2008
com.basis.bbj.comm.PortRequestServer.maxClients=500
com.basis.bbj.comm.PortRequestServer.start=true
com.basis.bbj.comm.ProxyManagerServer.bindAddr=127.0.0.1\:2009
com.basis.bbj.comm.ProxyManagerServer.maxClients=100
com.basis.bbj.comm.ProxyManagerServer.start=false
com.basis.bbj.comm.RuntimeMgr.notifyUserOnInternalError=false
com.basis.bbj.comm.RuntimeMgr.releaseOnLostConnection=true
com.basis.bbj.comm.RuntimeMgr.requireAllSendMsgResponses=false
com.basis.bbj.comm.RuntimeMgr.suppressPrinterAckBack=true
com.basis.bbj.comm.RuntimeMgr.suppressUIAckBack=true
com.basis.bbj.comm.RuntimeMgr.useDvkLicenseIfPresent=true
com.basis.bbj.comm.SecureThinClientServer.backLog=110
com.basis.bbj.comm.SecureThinClientServer.bindAddr=0.0.0.0\:2103
com.basis.bbj.comm.SecureThinClientServer.disallowConsole=true
com.basis.bbj.comm.SecureThinClientServer.maxClients=500
com.basis.bbj.comm.SecureThinClientServer.start=true
com.basis.bbj.comm.SecureThinClientServer.webUser=rti
com.basis.bbj.comm.TerminalServer.backLog=110
com.basis.bbj.comm.TerminalServer.bindAddr=127.0.0.1\:2004
com.basis.bbj.comm.TerminalServer.maxClients=500
com.basis.bbj.comm.TerminalServer.start=true
com.basis.bbj.comm.ThinClientProxyServer.backLog=110
com.basis.bbj.comm.ThinClientProxyServer.bindAddr=127.0.0.1\:2006
com.basis.bbj.comm.ThinClientProxyServer.err=/usr2/basis/log/ThinClientProxyServer.err
com.basis.bbj.comm.ThinClientProxyServer.keepLogs=7
com.basis.bbj.comm.ThinClientProxyServer.maxClients=500
com.basis.bbj.comm.ThinClientProxyServer.out=/usr2/basis/log/ThinClientProxyServer.out
com.basis.bbj.comm.ThinClientProxyServer.sizeM=10
com.basis.bbj.comm.ThinClientProxyServer.start=false
com.basis.bbj.comm.ThinClientProxyServer.waitTime=0
com.basis.bbj.comm.ThinClientServer.backLog=110
com.basis.bbj.comm.ThinClientServer.bindAddr=0.0.0.0\:2003
com.basis.bbj.comm.ThinClientServer.disallowConsole=true
com.basis.bbj.comm.ThinClientServer.encryptConnection=true
com.basis.bbj.comm.ThinClientServer.maxClients=500
com.basis.bbj.comm.ThinClientServer.start=false
com.basis.bbj.comm.ThinClientServer.translateAppletCmdLine=false
com.basis.bbj.comm.ThinClientServer.translateCmdLine=false
com.basis.bbj.comm.ThinClientServer.webUser=rti
com.basis.bbj.serverPinning=true
com.basis.bbj.sessionPinning=false
com.basis.filesystem.localPort=2000
com.basis.filesystem.localServer=127.0.0.1
com.basis.filesystem.remote.securedataserver.encrypter=com.basis.filesystem.remote.securedataserver.DefaultEncrypter
com.basis.server.LookAndFeel=default
com.basis.server.admin.1.defaultPort=2002
com.basis.server.admin.1.interface=0.0.0.0
com.basis.server.admin.1.start=true
com.basis.server.admin.1.useSSL=false
com.basis.server.admin.log=/usr2/basis/log/AdminServer.log
com.basis.server.allowPipe=true
com.basis.server.autorun.config=/usr2/bbx/config/config.bbx
com.basis.server.autorun.user=root
com.basis.server.autorun.workingDirectory=/usr2/bbx/
com.basis.server.configDirectory=/usr2/bbx/config
com.basis.server.exclusiveAccess=true
com.basis.server.filesystem.1.defaultPort=2000
com.basis.server.filesystem.1.defaultTimeout=2000
com.basis.server.filesystem.1.interface=0.0.0.0
com.basis.server.filesystem.1.prefixes=
com.basis.server.filesystem.1.start=true
com.basis.server.filesystem.1.useSSL=true
com.basis.server.filesystem.log=/usr2/basis/log/FilesystemServer.log
com.basis.server.memStatFrequence=15
com.basis.server.nio.override=false
com.basis.server.passwordAuth=false
com.basis.server.pro5ds.1.64bitMKeyed=false
com.basis.server.pro5ds.1.advisoryLocking=true
com.basis.server.pro5ds.1.defaultPort=2100
com.basis.server.pro5ds.1.defaultTimeout=5
com.basis.server.pro5ds.1.interface=0.0.0.0
com.basis.server.pro5ds.1.maskMKeyedFID=false
com.basis.server.pro5ds.1.prefixes=
com.basis.server.pro5ds.1.scanAllRoots=false
com.basis.server.pro5ds.1.start=false
com.basis.server.pro5ds.1.tagged=false
com.basis.server.pro5ds.1.umask=0000
com.basis.server.pro5ds.1.useSSL=true
com.basis.server.pro5ds.log=/usr2/basis/log/Pro5DSServer.log
com.basis.server.programCacheFile=/usr2/basis/cfg/BBjCache.txt
com.basis.server.programCacheSize=300
com.basis.server.resourceCacheFile=/usr2/basis//cfg/BBjResourceCache.txt
com.basis.server.skipProgramCacheFile=true
com.basis.server.sqlengine.1.debugLevel=2
com.basis.server.sqlengine.1.defaultPort=2001
com.basis.server.sqlengine.1.interface=0.0.0.0
com.basis.server.sqlengine.1.start=true
com.basis.server.sqlengine.1.useSSL=true
com.basis.server.sqlengine.log=/usr2/basis/log/SQLServer.log
com.basis.server.status.defaultPort=11057
com.basis.server.stdErr.keepLogs=7
com.basis.server.stdErr.sizeM=1
com.basis.server.stdErr=/usr2/basis/log/BBjServices.err
com.basis.server.stdOut.keepLogs=7
com.basis.server.stdOut.sizeM=1
com.basis.server.stdOut=/usr2/basis/log/BBjServices.out
com.basis.server.threadPoolSize=-1
com.basis.server.workingDirectory=/usr2/bbx/
com.basis.sql.logLevel=0
com.basis.user.useFork=true
default-config=/usr2/bbx/config/config.bbx
default-user=root
default-working-dir=/usr2/basis/bin
sun.arch.data.model=32
EOF
	close(BBJPROP);
	#Done creating BBj.properties


	# Basis License Manager.
	loginfo("Installing Basis License Manager...");
	if(! -f "/tmp/$BLM") {
		system("wget --proxy=off --cache=off -O /tmp/$BLM http://$TFSERVER/ks/$BLM");
		if(! -f "/tmp/$BLM") {
			logerror("Could not download file http://$TFSERVER/ks/$BBJ");
		}
	}
	if(-f "/tmp/$BLM") {
		$bbj_nozip = $BLM;
		$bbj_nozip =~ s/\.gz//g; # blah.jar.gz -> blah.jar
		system("gzip -dc /tmp/$BLM > /tmp/$bbj_nozip");
		system("java -jar /tmp/$bbj_nozip");
	}
	# Basis Admin to setup BBj in general (May need to manually do this)
	loginfo("Running Basis Admin Menu.....");
	system("/usr2/basis/bin/admin");
}


sub install_rtiv12_packages
{
	loginfo("Installing RTI v12 Software Dependencies...");
	loginfo("Using Teleflora Package Server \"$TFSERVER\"");


	remount_partition("/teleflora", "/usr2");


	loginfo("Installing Sendmail...");
	system("yum groupinstall -y mail-server");
	system("yum install -y fetchmail");

	if("$ARCH" eq "x86") {
	} else {
		system("rpm -ivh http://$TFSERVER/ks/ksh-20080202-14.el5.x86_64.rpm");
	}
}

sub install_daisyv8_packages
{
	my $dev = "";

	loginfo("Installing Daisy v8 Software Dependencies...");
	loginfo("Using Teleflora Package Server \"$TFSERVER\"");
	remount_partition("/teleflora", "/d");

	# Create a "/mnt/cdrom" item inside of /etc/fstab.
	system("grep \"\/mnt\/cdrom\" /etc/fstab > /dev/null 2> /dev/null");
	if ($? != 0) {
		open(FSTAB, ">> /etc/fstab");
		print(FSTAB "/dev/scd0\t\t/mnt/cdrom\tiso9660\tro,exec\t0 0\n");
		close(FSTAB);
	}

	unless (-d "/mnt/cdrom") {
		system("mkdir /mnt/cdrom");
	}

	unless (-d "/mnt/usb") {
		system("mkdir /mnt/usb");
	}
}



sub identify_hardware
{

	open(PIPE, "/usr/sbin/dmidecode |");
	while(<PIPE>) {
		if(/PowerEdge T300/) {
			$HARDWARE = "t300";
		} elsif (/PowerEdge T310/) {
			$HARDWARE = "t310";
		} elsif (/PowerEdge T410/) {
			$HARDWARE = "t410";
		} elsif (/PowerEdge 1800/) {
			$HARDWARE = "1800";
		} elsif (/PowerEdge 420/) {
			$HARDWARE = "420";
		} elsif (/PowerEdge R910/) {
			$HARDWARE = "r910";
		} elsif (/PowerEdge 2950/) {
			$HARDWARE = "2950";
		} elsif (/Precision 380/) {
			$HARDWARE = "Precision380";
		} elsif (/Precision 390/) {
			$HARDWARE = "Precision390";
		} elsif (/Precision T3400/) {
			$HARDWARE = "t3400";
		} elsif (/Precision T3500/) {
			$HARDWARE = "t3500";
		} else {
			$HARDWARE = "unknown";
		}
	}
	close(PIPE);
}



#
# By default, redhat will mount partitions based on a label name, even swap partitions.
# We are using 'dd' to "clone" disks from sda to sdb, which means, two disks will have a partition
# labeled the same thing.
# This patch modifies /etc/fstab to remove the "label based mounting" and instead, use 
# "hard coded device names".
#
sub remount_partition
{
	my $oldlabel = $_[0];
	my $newmountpoint = $_[1];
	my $oldmountpoint = "";
	my $device  = "";
	my $line = "";
	my @array = ();


	# LABEL=SWAP-sda2 was "swap" mounted to "/dev/sda2"
	if("$oldlabel" =~ /(SWAP)(-)(\S+)/) {
		$device = "/dev/$3";
		$oldmountpoint = "swap";

	# Which device is LABEL=/teleflora mounted to currently ?
	} else {

		open(PIPE, "mount -l |");
		while(<PIPE>) {
			# e2fs labels show up with [labelname]
			next until /\[$oldlabel\]/;

			@array = split(/\s+/);
			$device = $array[0];
			$oldmountpoint = $array[2];
			last;
		}
		close(PIPE);
	}

	if("$device" eq "") {
		return(0);
	}
	if("$oldmountpoint" eq "") {
		return(0);
	}


	# We did not find this label in /etc/fstab. We're done.
	system("grep \"LABEL=$oldlabel\" /etc/fstab > /dev/null 2> /dev/null");
	if($? != 0) {
		return(0);
	}


	# Create our new mountpoint.
	loginfo("Mounting \"$device-> $newmountpoint\" instead of \"LABEL=$oldlabel -> $oldmountpoint\" ...");
	if("$newmountpoint" eq "swap") {
		system("swapoff $device");
	} elsif ("$newmountpoint" ne "/") {
		system("umount $oldmountpoint");
	}



	# Modify /etc/fstab
	open(OLD, "< /etc/fstab");
	open(NEW, "> /etc/fstab.$$");
	while(<OLD>) {
		if(/^(\s*)(LABEL)(\s*)(=)(\s*)($oldlabel)(\s+)/) {
			@array = split(/\s+/);
			print(NEW "$device\t$newmountpoint\t$array[2]\t$array[3]\t$array[4] $array[5]\n");
			next;
		}
		print(NEW);
	}
	close(OLD);
	close(NEW);
	system("mv /etc/fstab.$$ /etc/fstab");

	# In the case of our root partition, we also need to modify grub.conf
	if($oldlabel eq "/") {
		open(OLD, "< /boot/grub/grub.conf");
		open(NEW, "> /boot/grub/grub.conf.$$");
		while(<OLD>) {
			$line = $_;

			#root=LABEL=/ -> root=/dev/blah
			if(/(root=LABEL=)(\s*)($oldlabel)/) {
				$line =~ s/(root=LABEL=)(\s*)($oldlabel)/root=$device/g;
				print(NEW $line);
				next;
			}
			print(NEW);
		}
		close(OLD);
		close(NEW);
		system("mv /boot/grub/grub.conf.$$ /boot/grub/grub.conf");
	}



	# Make sure our mountpoint exists.
	if( ("$newmountpoint" ne "swap")
	&&  (! -d "$newmountpoint") ) {
		system("mkdir -p $newmountpoint");
	}

	# Re-mount the device.
	if("$newmountpoint" eq "swap") {
		system("swapon $device");
	} else {
		if("$newmountpoint" ne "/") {
			system("mount $newmountpoint");
		}
	}

	return(0);
}



sub reset_aide_database
{

	if(! -f "/usr/sbin/aide") {
		return(1);
	}

	loginfo("Resetting Intrusion Detection Rules...");
	if(-f "/var/lib/aide/aide.db.gz") {
                $pid = fork();
                if ($pid == 0) {
                        loginfo("Update Aide Databases.");
                        system("/usr/sbin/aide --init");
                        system("mv /var/lib/aide/aide.db.new.gz /var/lib/aide/aide.db.gz");
                        exit(0);
                } else {
                        loginfo("Update Aide Database stated as PID $pid at ".
localtime() . "\n");
                }
                exit(0);
        }

	return(0);
}





#
# Unmount all filesystems except '/'
# Useful if we need to perform a "power button" shutdown, to ensure that most of our 
# filesystems don't come up in a 'dirty' state.
#
# Notes:
# We explicity do not run 'sync' as, I have experienced in the past, cases where 'sync' hung,
# I believe because the backup device could not sync, and, "sync" works on all devices.
# The filesystem caches are all sycned when the FS is unmounted.
# This function should be run as root.
#
sub hard_reboot
{
	my @array = ();
	my $mountpoint = "";
	my $device = "";
	my $pid = -1;
	my $childpid = -1;
	my $i = 0;


	loginfo("Performing reboot\n");

	# Run the "regular" shutdown here. If that works as expected, we should never actually
	# finish this function, as, we will be killed off.
	$pid = fork();
	if($pid == 0) {
		exec("/sbin/shutdown -r now");
	}
	for($i = 0; $i < 120; $i++) {
		$childpid = waitpid($pid, WNOHANG);
		last if($childpid != 0);
		sleep(1);
	}
	if($childpid < 0) {
		logerror("waitpid() on PID $pid returned $childpid. This is odd.");
	}

	if($i >= 120) {
		logerror("We waited long enough (2 minutes) for our shutdown to finish. Will manually reboot.");
		system("kill -TERM $pid");
	} else {
		logerror("This is odd. Our 'shutdown' child terminated, and yet, we have not been killed.");
		return;
	}

	# If we get this far, though "shutdown" ran, it never actually did anything.
	# We have seen this in the field in cases where sync() hangs (and hence, the 'shutdown' process 
	# also hangs.
	# At this point, we will manually unmount disks, kill off processes, and reboot the computer.
	# Note! Make sure *not* to run 'sync', as, that will almost certainly hang.
	loginfo("Server did not reboot normally. Will issue a 'hard' reboot.");
	print("Server did not reboot normally. Will issue a 'hard' reboot.\n");
	sleep(5);

	# Bring down
	system("/sbin/service smb stop");
	system("/sbin/service sendmail stop");
	if (-f "/etc/init.d/rti") {
		system("/sbin/service rti stop");
	}
	if(-f "/etc/init.d/daisy") {
		system("/sbin/service daisy stop");
	}


	# If things are this un-healthy, then, we may have underlying disk issues.
	# If we simply reboot without the '/fsckoptions' file, then, chances are, the flower shop
	# owner will be presented with an ugly prompt saying 'fsck failed', and a '#' prompt.
	# By adding this particular file, according to /etc/rc.d/rc.sysinit, we FSCK will use these "force" and "dammit"
	# options, thereby sparing the florist from having to type stuff in.
	open(FILE, "> /fsckoptions");
		print(FILE "-f -v -y");
	close(FILE);

	open(MOUNTS, "< /proc/mounts");
	while(<MOUNTS>) {
		chomp;
		next until(/ext[23]/);
		@array = split(/\s+/);
		$device = $array[0];
		$mountpoint = $array[1];
		next if(! $device);
		next if("$device" eq "");
		next if("$mountpoint" eq "");
		next if("$mountpoint" eq "/");
		next if("$mountpoint" eq "/proc");
		next if(! -b $device);

		# Look at all processes holding this filesystem open, and kill them.
		#[kachline@rtidev bin]$ sudo /usr/sbin/lsof /usr4
		#COMMAND  PID USER   FD   TYPE DEVICE SIZE   NODE NAME
		#bash    4283   jj  cwd    DIR  104,2 4096 961291 /usr4/jj/rev
		print("Killing Processes ($device) ....\n");
		open(LSOF, "/usr/sbin/lsof $device |");
		while(<LSOF>) {
			chomp;
			@array = split(/\s+/);
			$pid = $array[1];
			next if(! $pid);
			next if("$pid" eq "PID");
			print("$_\n");
			system("kill -HUP $pid");
			system("kill -TERM  $pid");
			system("kill -INT $pid");
			system("kill -KILL $pid");
		}
		close(LSOF);

		# Unmount the filesystem
		print("Unmounting $mountpoint -> $device");
		system("umount $device");
		if($? != 0) {
			print("Forcing Unmount of $mountpoint -> $device");
			system("umount -f $device");
			if($? != 0) {
				print("Error: Could not unmount $mountpoint -> $device");
				print("Could not Unmount \"$device\".\n");
				print("Try re-running this script to kill off 'lsof' processes.\n");
			}
		}

	}
	close(MOUNTS);

	system("/sbin/reboot -n -f");
}



#
# Many of our original T300 servers came shipped with a swap partitioin in place, yet, a bug in /etc/fstab
# which would cause the swap partition to never "swapon".
# This routing fixes said bug.
#
sub swapon
{
	my @partitions = ();
	my @array = ();
	my $thisdisk = "";
	my $thispart = "";


	# only RTI systems
	if(-d "/d/daisy") {
		logerror("This is a daisy system. Will not update swap space.\n");
		return (0);
	}


	# Are we root?
	if($UID != 0) {
		logerror("Error: Must be root to run.\n");
		return(1);
	}


	# Which partitions are formatted for swap?
	foreach $thisdisk ("/dev/sda", "/dev/sdb") {
		open(FDISK, "/sbin/fdisk -l $thisdisk |");
		while(<FDISK>) {
			next until(/^\/dev\//);
			next unless(/(Linux)(\s+)(swap)/);
			@array = split(/\s+/);
			$thispart = $array[0];
			if(-b "$thispart") {
				push(@partitions, $thispart);
			}
		}
		close(FDISK);
	}


	# Did we find swap partitions?
	if($#partitions < 0) {
		logerror("Error: No swap partitions were found! No changes made.");
		return(5);
	}


	# Modify /etc/fstab
	loginfo("Updating /etc/fstab...");
	open(OLD, "< /etc/fstab") or die("Could not open /etc/fstab for read");
	open(NEW, "> /etc/fstab.$$") or die("Could not open /etc/fstab.$$ for write");
	while(<OLD>) {
		next if(/(\s+)(swap)(\s+)/);
		print(NEW);
	}
	close(OLD);

	# Record our known swap partitions.
	foreach $thispart (sort(@partitions)) {
		print(NEW "$thispart\t\tswap\t\tswap\tdefaults\t\t0\t0\n");
	}
	close(NEW);

	if(-s "/etc/fstab.$$" <= 0) {
		# We probably didn't have permission to create this temp file.
		logerror("Our newly created fstab file does not exist. No changes made.");
		return(6);
	}
	system("mv /etc/fstab.$$ /etc/fstab");


	# Format each swap partition.
	foreach $thispart (sort(@partitions)) {
		loginfo("Enabling Swap Partition \"$thispart\" ...");
		system("/sbin/mkswap $thispart");
		system("/sbin/swapon $thispart");
	}
	close(NEW);

	# Enable our swap partitions.
	system("cat /proc/meminfo | grep Swap");

	loginfo("Swap Space Updated.\n");
	return(0);
}


#
# Remove files from /var/spool/cups/tmp
# Earlier versions of RTI would leave behind lots of zero byte files in 
# this directory.
#
sub cups_clean_tempfiles
{
	system("find /var/spool/cups/tmp -type f -print | xargs rm -f");
	return(0);
}


#
# Edit the CUPS conf file.
#
sub cups_edit_conf
{
	my $timestamp = strftime("%Y-%m-%d_%H%M%S", localtime());
	my $conf_file = '/etc/cups/cupsd.conf';
	my $new_conf_file = "$conf_file-$timestamp";

	unless (-f "$conf_file") {
		logerror("Error: $conf_file not found");
		return(1);
	}

	unless (open(OLD, "< $conf_file")) {
		showerror("Could not open $conf_file for read");
		return(2);
	}

	unless (open(NEW, "> $new_conf_file")) {
		showerror("Could not open $new_conf_file for write");
		close(OLD);
		return(3);
	}

	while(<OLD>) {

		next if (/^#updateos.pl#/);

		# CUPS has a default "max jobs" set to 500. For some of the
		# larger shops, this print queue size is too small.
		# Set the "Max Jobs" setting to a higher value.

		if ($CUPSCONF_MAXJOBS) {
			if(/^(\s*)(MaxJobs)(\s+)/) {
				print(NEW "#updateos.pl# $_");
				next;
			}
		}


		#
		# Don't put files into /var/spool/cups/tmp at all.
		# Thus eliminating the need for the "cleanfiles"
		# patch elsewhere in this script.
		#
		if ($CUPSCONF_DISABLETEMPFILES) {
			if (/^(\s*)(PreserveJobHistory)(\s+)/i) {
				print(NEW "#updateos.pl# $_");
				next;
			}
			if (/^(\s*)(PreserveJobFiles)(\s+)/i) {
				print(NEW "#updateos.pl# $_");
				next;
			}
		}

		if ($CUPSCONF_TIMEOUTS) {
			if (/^(\s*)(BrowseTimeout)(\s+)/i) {
				print(NEW "#updateos.pl# $_");
				next;
			}

			if (/^(\s*)(Timeout)(\s+)/i) {
				print(NEW "#updateos.pl# $_");
				next;
			}
		}

		if ($CUPSCONF_ERRORPOLICY) {
			if (/^(\s*)(ErrorPolicy)(\s+)/i) {
				print(NEW "#updateos.pl# $_");
				next;
			}
		}
		print(NEW);
	};

	close(OLD);

	loginfo("Adding rules to $conf_file");
	print(NEW "#updateos.pl# --- Start of additional lines ---\n");
	print(NEW "#updateos.pl# $0 " . '$Revision: 1.99 $' . " $timestamp\n");
	print(NEW "MaxJobs 10000\n");
	print(NEW "PreserveJobHistory No\n");
	print(NEW "PreserveJobFiles No\n");
	print(NEW "BrowseTimeout 0\n");
	print(NEW "Timeout 0\n");
	print(NEW "ErrorPolicy retry-job\n");
	print(NEW "#updateos.pl# --- End of additional lines ---\n");

	close(NEW);

	# If we created a new conf file that is zero sized, that is bad.
	if (-z $new_conf_file) {
		logerror("Error: Copy of $conf_file is a zero size file");
		system("rm $new_conf_file");
		return(4);
	}

	# Assume conf file was successfully transformed...
	# so replace the old one with the new.

	system("chmod --reference=$conf_file $new_conf_file");
	system("chown --reference=$conf_file $new_conf_file");
	system("cp $new_conf_file $conf_file");

	system("/sbin/service cups start");

	return(0);
}


#
# Remove all print jobs from all print queues.
#
sub purge_printjobs
{
	system("/sbin/service cups stop");
	system("find /var/spool/cups -type f -print | xargs rm -f");
	system("/sbin/service cups start");
	return(0);
}


#
# Gather some evidence if we believe there was a compromise or need performance
# statistics.
#
sub forensics
{
	my $pid = "";
	my @array = ();


	unlink("/tmp/system-info.$$");

	system("/bin/echo '----- Basic Information -----' | tee --append /tmp/system-info.$$");
	system("date >> /tmp/system-info.$$");
	system("hostname >> /tmp/system-info.$$");


	system("/bin/echo '----- Network Information -----' | tee --append /tmp/system-info.$$");
	system("/sbin/ifconfig >> /tmp/system-info.$$");
	system("/bin/netstat -plan >> /tmp/system-info.$$");
	system("/sbin/route -n >> /tmp/system-info.$$");

	#
	# Run nmap discovery on all networks which we have a (known) route to.
	#
	open(PIPE, "/sbin/route -n |");
	while(<PIPE>) {
		if(/^\d/) {
			@array = split(/(\s+)/);
			next if("$array[0]" eq "0.0.0.0");
			next if("$array[0]" eq "127.0.0.0");
			next if("$array[0]" !~ /^\d/);
			system("/bin/echo '----- nmap $array[0] -----' | tee --append /tmp/system-info.$$");
			system("nmap -v -n -sS -O $array[0]/24 >> /tmp/system-info.$$");
		}
	}
	close(PIPE);


	system("/bin/echo '----- Processes (ps wwaux) -----' | tee --append /tmp/system-info.$$");
	system("ps wwaux >> /tmp/system-info.$$");

	# We may not be able to trust 'ps' results.
	system("/bin/echo '----- Processes (/proc) -----' | tee --append /tmp/system-info.$$");
	open(PROCS, "find /proc -type d -print |");
	while(<PROCS>) {
		chomp;
		next until (/\d+/);
		$pid = $_;
		print("$pid: ");
		system("/bin/echo \"/proc/$pid/cmdline\" >> /tmp/system-info.$$");
		system("cat /proc/$pid/cmdline >> /tmp/system-info.$$");
	}
	close(PROCS);


	system("/bin/echo '----- System Performance -----' | tee --append /tmp/system-info.$$");
	system("cat /proc/meminfo >> /tmp/system-info.$$");
	system("vmstat 2 5 >> /tmp/system-info.$$");
	system("iostat -x 2 5 >> /tmp/system-info.$$");
	system("/usr/sbin/dmidecode >> /tmp/system-info.$$");


	system("/bin/echo '----- OS Information -----' | tee --append /tmp/system-info.$$");
	system("uname -a >> /tmp/system-info.$$");
	system("cat /proc/redhat-release >> /tmp/system-info.$$");
	system("rpm --list >> /tmp/system-info.$$");
	system("rpm --verify >> /tmp/system-info.$$");
	system("dmesg >> /tmp/system-info.$$");
	if(-f "/usr/bin/yum") {
		system("yum checkupdate >> /tmp/system-info.$$");
	}



	system("/bin/echo '----- User Information -----' | tee --append /tmp/system-info.$$");
	system("w >> /tmp/system-info.$$");
	system("last >> /tmp/system-info.$$");
	system("/usr2/bbx/bin/rtiuser.pl --list >> /tmp/system-info.$$");

	system("/bin/echo '----- PCI 2.2.2 -----' | tee --append /tmp/system-info.$$");
	system("/sbin/chkconfig --list | grep ':on' >> /tmp/system-info.$$");

	system("/bin/echo '----- PCI 2.2.3 -----' | tee --append /tmp/system-info.$$");
	system("/sbin/chkconfig --list | grep ':on' >> /tmp/system-info.$$");



	# Create our resultant tarfile.
	unlink("/tmp/system-info.$$.tar");
	system("cd /tmp && tar --append -vf /tmp/system-info.$$.tar system-info.$$");
	system("cd / && tar --append -vf /tmp/system-info.$$.tar /etc");
	system("cd /home && find . -type f -iname .bash_history -exec tar --append -vf /tmp/system-info.$$.tar \\{\\} \\;");
	system("cd / && find /root -type f -iname .bash_history -exec tar --append -vf /tmp/system-info.$$.tar \\{\\} \\;");
	system("cd / && tar --append -vf /tmp/system-info.$$.tar root");
	system("cd / && tar --append -vf /tmp/system-info.$$.tar var/log");
	if(-d "/usr2/bbx") {
		system("cd / && tar --append -vf /tmp/system-info.$$.tar usr2/bbx/log");
		system("cd / && tar --append -vf /tmp/system-info.$$.tar usr2/bbx/config");
	}
	if(-d "/d/daisy") {
		system("cd / && tar --append -vf /tmp/system-info.$$.tar d/daisy/log");
	}

	system("gzip -v9 /tmp/system-info.$$.tar");
	system("ls -la /tmp/system-info.$$.tar.gz");


	return(0);
}


sub loginfo
{
	my $message = $_[0];
	my $logtime = strftime("%Y-%m-%d %H:%M:%S", localtime());

	chomp($message);
	if("$POSDIR" ne "" && -d "$POSDIR") {
		open LOGFILE, ">> $POSDIR/log/RTI-Patches.log";
			print LOGFILE "$logtime ($0-$$) <I> $message\r\n";
		close LOGFILE;
	}
	print("$message\n");


	return "";
}


sub logerror
{
	my $message = $_[0];
	my $logtime = strftime("%Y-%m-%d %H:%M:%S", localtime());

	chomp($message);
	if("$POSDIR" ne "") {
		open LOGFILE, ">> $POSDIR/log/RTI-Patches.log";
			print LOGFILE "$logtime ($0-$$) <E> $message\r\n";
		close LOGFILE;
	}
	print("$message\n");

	return "";
}


